# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.network_interconnects import setting_update_params
from ...types.network_interconnects.setting_get_response import SettingGetResponse
from ...types.network_interconnects.setting_update_response import SettingUpdateResponse

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        default_asn: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingUpdateResponse:
        """
        Update the current settings for the active account

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/cni/settings", account_id=account_id),
            body=maybe_transform({"default_asn": default_asn}, setting_update_params.SettingUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingUpdateResponse,
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingGetResponse:
        """
        Get the current settings for the active account

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/cni/settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingGetResponse,
        )


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        account_id: str,
        default_asn: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingUpdateResponse:
        """
        Update the current settings for the active account

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/cni/settings", account_id=account_id),
            body=await async_maybe_transform({"default_asn": default_asn}, setting_update_params.SettingUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingUpdateResponse,
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingGetResponse:
        """
        Get the current settings for the active account

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/cni/settings", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingGetResponse,
        )


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.update = to_raw_response_wrapper(
            settings.update,
        )
        self.get = to_raw_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.update = async_to_raw_response_wrapper(
            settings.update,
        )
        self.get = async_to_raw_response_wrapper(
            settings.get,
        )


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.update = to_streamed_response_wrapper(
            settings.update,
        )
        self.get = to_streamed_response_wrapper(
            settings.get,
        )


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.update = async_to_streamed_response_wrapper(
            settings.update,
        )
        self.get = async_to_streamed_response_wrapper(
            settings.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ....._models import BaseModel

__all__ = ["FilterCreateResponse"]


class FilterCreateResponse(BaseModel):
    id: str
    """The unique ID of the expression filter."""

    created_on: datetime
    """The creation timestamp of the expression filter."""

    expression: str
    """The filter expression."""

    mode: str
    """The filter's mode. Must be one of 'enabled', 'disabled', 'monitoring'."""

    modified_on: datetime
    """The last modification timestamp of the expression filter."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .route import Route as Route
from .teamnet import Teamnet as Teamnet
from .hostname_route import HostnameRoute as HostnameRoute
from .virtual_network import VirtualNetwork as VirtualNetwork
from .route_edit_params import RouteEditParams as RouteEditParams
from .route_list_params import RouteListParams as RouteListParams
from .subnet_list_params import SubnetListParams as SubnetListParams
from .route_create_params import RouteCreateParams as RouteCreateParams
from .hostname_route_edit_params import HostnameRouteEditParams as HostnameRouteEditParams
from .hostname_route_list_params import HostnameRouteListParams as HostnameRouteListParams
from .virtual_network_edit_params import VirtualNetworkEditParams as VirtualNetworkEditParams
from .virtual_network_list_params import VirtualNetworkListParams as VirtualNetworkListParams
from .hostname_route_create_params import HostnameRouteCreateParams as HostnameRouteCreateParams
from .virtual_network_create_params import VirtualNetworkCreateParams as VirtualNetworkCreateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .v2 import (
    V2Resource,
    AsyncV2Resource,
    V2ResourceWithRawResponse,
    AsyncV2ResourceWithRawResponse,
    V2ResourceWithStreamingResponse,
    AsyncV2ResourceWithStreamingResponse,
)
from .direct_uploads import (
    DirectUploadsResource,
    AsyncDirectUploadsResource,
    DirectUploadsResourceWithRawResponse,
    AsyncDirectUploadsResourceWithRawResponse,
    DirectUploadsResourceWithStreamingResponse,
    AsyncDirectUploadsResourceWithStreamingResponse,
)

__all__ = [
    "DirectUploadsResource",
    "AsyncDirectUploadsResource",
    "DirectUploadsResourceWithRawResponse",
    "AsyncDirectUploadsResourceWithRawResponse",
    "DirectUploadsResourceWithStreamingResponse",
    "AsyncDirectUploadsResourceWithStreamingResponse",
    "V2Resource",
    "AsyncV2Resource",
    "V2ResourceWithRawResponse",
    "AsyncV2ResourceWithRawResponse",
    "V2ResourceWithStreamingResponse",
    "AsyncV2ResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CommandCreateParams", "Command", "CommandCommandArgs"]


class CommandCreateParams(TypedDict, total=False):
    account_id: Required[str]

    commands: Required[Iterable[Command]]
    """List of device-level commands to execute"""


class CommandCommandArgs(TypedDict, total=False):
    interfaces: List[Literal["default", "tunnel"]]
    """List of interfaces to capture packets on"""

    max_file_size_mb: Annotated[float, PropertyInfo(alias="max-file-size-mb")]
    """Maximum file size (in MB) for the capture file.

    Specifies the maximum file size of the warp-diag zip artifact that can be
    uploaded. If the zip artifact exceeds the specified max file size, it will NOT
    be uploaded
    """

    packet_size_bytes: Annotated[float, PropertyInfo(alias="packet-size-bytes")]
    """Maximum number of bytes to save for each packet"""

    test_all_routes: Annotated[bool, PropertyInfo(alias="test-all-routes")]
    """Test an IP address from all included or excluded ranges.

    Tests an IP address from all included or excluded ranges. Essentially the same
    as running 'route get <ip>'' and collecting the results. This option may
    increase the time taken to collect the warp-diag
    """

    time_limit_min: Annotated[float, PropertyInfo(alias="time-limit-min")]
    """Limit on capture duration (in minutes)"""


class Command(TypedDict, total=False):
    command_type: Required[Literal["pcap", "warp-diag"]]
    """Type of command to execute on the device"""

    device_id: Required[str]
    """Unique identifier for the physical device"""

    user_email: Required[str]
    """Email tied to the device"""

    command_args: CommandCommandArgs

    registration_id: str
    """Unique identifier for the device registration.

    Required for multi-user devices to target the correct user session.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .scim.scim import (
    SCIMResource,
    AsyncSCIMResource,
    SCIMResourceWithRawResponse,
    AsyncSCIMResourceWithRawResponse,
    SCIMResourceWithStreamingResponse,
    AsyncSCIMResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from .access_requests import (
    AccessRequestsResource,
    AsyncAccessRequestsResource,
    AccessRequestsResourceWithRawResponse,
    AsyncAccessRequestsResourceWithRawResponse,
    AccessRequestsResourceWithStreamingResponse,
    AsyncAccessRequestsResourceWithStreamingResponse,
)

__all__ = ["LogsResource", "AsyncLogsResource"]


class LogsResource(SyncAPIResource):
    @cached_property
    def access_requests(self) -> AccessRequestsResource:
        return AccessRequestsResource(self._client)

    @cached_property
    def scim(self) -> SCIMResource:
        return SCIMResource(self._client)

    @cached_property
    def with_raw_response(self) -> LogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LogsResourceWithStreamingResponse(self)


class AsyncLogsResource(AsyncAPIResource):
    @cached_property
    def access_requests(self) -> AsyncAccessRequestsResource:
        return AsyncAccessRequestsResource(self._client)

    @cached_property
    def scim(self) -> AsyncSCIMResource:
        return AsyncSCIMResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLogsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLogsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLogsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLogsResourceWithStreamingResponse(self)


class LogsResourceWithRawResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

    @cached_property
    def access_requests(self) -> AccessRequestsResourceWithRawResponse:
        return AccessRequestsResourceWithRawResponse(self._logs.access_requests)

    @cached_property
    def scim(self) -> SCIMResourceWithRawResponse:
        return SCIMResourceWithRawResponse(self._logs.scim)


class AsyncLogsResourceWithRawResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

    @cached_property
    def access_requests(self) -> AsyncAccessRequestsResourceWithRawResponse:
        return AsyncAccessRequestsResourceWithRawResponse(self._logs.access_requests)

    @cached_property
    def scim(self) -> AsyncSCIMResourceWithRawResponse:
        return AsyncSCIMResourceWithRawResponse(self._logs.scim)


class LogsResourceWithStreamingResponse:
    def __init__(self, logs: LogsResource) -> None:
        self._logs = logs

    @cached_property
    def access_requests(self) -> AccessRequestsResourceWithStreamingResponse:
        return AccessRequestsResourceWithStreamingResponse(self._logs.access_requests)

    @cached_property
    def scim(self) -> SCIMResourceWithStreamingResponse:
        return SCIMResourceWithStreamingResponse(self._logs.scim)


class AsyncLogsResourceWithStreamingResponse:
    def __init__(self, logs: AsyncLogsResource) -> None:
        self._logs = logs

    @cached_property
    def access_requests(self) -> AsyncAccessRequestsResourceWithStreamingResponse:
        return AsyncAccessRequestsResourceWithStreamingResponse(self._logs.access_requests)

    @cached_property
    def scim(self) -> AsyncSCIMResourceWithStreamingResponse:
        return AsyncSCIMResourceWithStreamingResponse(self._logs.scim)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["StatusEditResponse"]


class StatusEditResponse(BaseModel):
    status: Literal["queued", "running", "paused", "errored", "terminated", "complete", "waitingForPause", "waiting"]

    timestamp: datetime
    """Accepts ISO 8601 with no timezone offsets and in UTC."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.gateway import (
    certificate_create_params,
    certificate_activate_params,
    certificate_deactivate_params,
)
from ....types.zero_trust.gateway.certificate_get_response import CertificateGetResponse
from ....types.zero_trust.gateway.certificate_list_response import CertificateListResponse
from ....types.zero_trust.gateway.certificate_create_response import CertificateCreateResponse
from ....types.zero_trust.gateway.certificate_delete_response import CertificateDeleteResponse
from ....types.zero_trust.gateway.certificate_activate_response import CertificateActivateResponse
from ....types.zero_trust.gateway.certificate_deactivate_response import CertificateDeactivateResponse

__all__ = ["CertificatesResource", "AsyncCertificatesResource"]


class CertificatesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CertificatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CertificatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CertificatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CertificatesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        validity_period_days: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateCreateResponse]:
        """
        Create a new Zero Trust certificate.

        Args:
          validity_period_days: Sets the certificate validity period in days (range: 1-10,950 days / ~30 years).
              Defaults to 1,825 days (5 years). **Important**: This field is only settable
              during the certificate creation. Certificates becomes immutable after creation -
              use the `/activate` and `/deactivate` endpoints to manage certificate lifecycle.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/gateway/certificates", account_id=account_id),
            body=maybe_transform(
                {"validity_period_days": validity_period_days}, certificate_create_params.CertificateCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateCreateResponse]], ResultWrapper[CertificateCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[CertificateListResponse]:
        """
        List all Zero Trust certificates for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/certificates", account_id=account_id),
            page=SyncSinglePage[CertificateListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CertificateListResponse,
        )

    def delete(
        self,
        certificate_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateDeleteResponse]:
        """Delete a gateway-managed Zero Trust certificate.

        You must deactivate the
        certificate from the edge (inactive) before deleting it.

        Args:
          certificate_id: Identify the certificate with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/gateway/certificates/{certificate_id}",
                account_id=account_id,
                certificate_id=certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateDeleteResponse]], ResultWrapper[CertificateDeleteResponse]),
        )

    def activate(
        self,
        certificate_id: str,
        *,
        account_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateActivateResponse]:
        """
        Bind a single Zero Trust certificate to the edge.

        Args:
          certificate_id: Identify the certificate with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/gateway/certificates/{certificate_id}/activate",
                account_id=account_id,
                certificate_id=certificate_id,
            ),
            body=maybe_transform(body, certificate_activate_params.CertificateActivateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateActivateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateActivateResponse]], ResultWrapper[CertificateActivateResponse]),
        )

    def deactivate(
        self,
        certificate_id: str,
        *,
        account_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateDeactivateResponse]:
        """
        Unbind a single Zero Trust certificate from the edge.

        Args:
          certificate_id: Identify the certificate with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/gateway/certificates/{certificate_id}/deactivate",
                account_id=account_id,
                certificate_id=certificate_id,
            ),
            body=maybe_transform(body, certificate_deactivate_params.CertificateDeactivateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateDeactivateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateDeactivateResponse]], ResultWrapper[CertificateDeactivateResponse]),
        )

    def get(
        self,
        certificate_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateGetResponse]:
        """
        Get a single Zero Trust certificate.

        Args:
          certificate_id: Identify the certificate with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/gateway/certificates/{certificate_id}",
                account_id=account_id,
                certificate_id=certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateGetResponse]], ResultWrapper[CertificateGetResponse]),
        )


class AsyncCertificatesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCertificatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCertificatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCertificatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCertificatesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        validity_period_days: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateCreateResponse]:
        """
        Create a new Zero Trust certificate.

        Args:
          validity_period_days: Sets the certificate validity period in days (range: 1-10,950 days / ~30 years).
              Defaults to 1,825 days (5 years). **Important**: This field is only settable
              during the certificate creation. Certificates becomes immutable after creation -
              use the `/activate` and `/deactivate` endpoints to manage certificate lifecycle.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/gateway/certificates", account_id=account_id),
            body=await async_maybe_transform(
                {"validity_period_days": validity_period_days}, certificate_create_params.CertificateCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateCreateResponse]], ResultWrapper[CertificateCreateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CertificateListResponse, AsyncSinglePage[CertificateListResponse]]:
        """
        List all Zero Trust certificates for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/certificates", account_id=account_id),
            page=AsyncSinglePage[CertificateListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CertificateListResponse,
        )

    async def delete(
        self,
        certificate_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateDeleteResponse]:
        """Delete a gateway-managed Zero Trust certificate.

        You must deactivate the
        certificate from the edge (inactive) before deleting it.

        Args:
          certificate_id: Identify the certificate with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/gateway/certificates/{certificate_id}",
                account_id=account_id,
                certificate_id=certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateDeleteResponse]], ResultWrapper[CertificateDeleteResponse]),
        )

    async def activate(
        self,
        certificate_id: str,
        *,
        account_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateActivateResponse]:
        """
        Bind a single Zero Trust certificate to the edge.

        Args:
          certificate_id: Identify the certificate with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/gateway/certificates/{certificate_id}/activate",
                account_id=account_id,
                certificate_id=certificate_id,
            ),
            body=await async_maybe_transform(body, certificate_activate_params.CertificateActivateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateActivateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateActivateResponse]], ResultWrapper[CertificateActivateResponse]),
        )

    async def deactivate(
        self,
        certificate_id: str,
        *,
        account_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateDeactivateResponse]:
        """
        Unbind a single Zero Trust certificate from the edge.

        Args:
          certificate_id: Identify the certificate with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/gateway/certificates/{certificate_id}/deactivate",
                account_id=account_id,
                certificate_id=certificate_id,
            ),
            body=await async_maybe_transform(body, certificate_deactivate_params.CertificateDeactivateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateDeactivateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateDeactivateResponse]], ResultWrapper[CertificateDeactivateResponse]),
        )

    async def get(
        self,
        certificate_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CertificateGetResponse]:
        """
        Get a single Zero Trust certificate.

        Args:
          certificate_id: Identify the certificate with a UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not certificate_id:
            raise ValueError(f"Expected a non-empty value for `certificate_id` but received {certificate_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/gateway/certificates/{certificate_id}",
                account_id=account_id,
                certificate_id=certificate_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CertificateGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CertificateGetResponse]], ResultWrapper[CertificateGetResponse]),
        )


class CertificatesResourceWithRawResponse:
    def __init__(self, certificates: CertificatesResource) -> None:
        self._certificates = certificates

        self.create = to_raw_response_wrapper(
            certificates.create,
        )
        self.list = to_raw_response_wrapper(
            certificates.list,
        )
        self.delete = to_raw_response_wrapper(
            certificates.delete,
        )
        self.activate = to_raw_response_wrapper(
            certificates.activate,
        )
        self.deactivate = to_raw_response_wrapper(
            certificates.deactivate,
        )
        self.get = to_raw_response_wrapper(
            certificates.get,
        )


class AsyncCertificatesResourceWithRawResponse:
    def __init__(self, certificates: AsyncCertificatesResource) -> None:
        self._certificates = certificates

        self.create = async_to_raw_response_wrapper(
            certificates.create,
        )
        self.list = async_to_raw_response_wrapper(
            certificates.list,
        )
        self.delete = async_to_raw_response_wrapper(
            certificates.delete,
        )
        self.activate = async_to_raw_response_wrapper(
            certificates.activate,
        )
        self.deactivate = async_to_raw_response_wrapper(
            certificates.deactivate,
        )
        self.get = async_to_raw_response_wrapper(
            certificates.get,
        )


class CertificatesResourceWithStreamingResponse:
    def __init__(self, certificates: CertificatesResource) -> None:
        self._certificates = certificates

        self.create = to_streamed_response_wrapper(
            certificates.create,
        )
        self.list = to_streamed_response_wrapper(
            certificates.list,
        )
        self.delete = to_streamed_response_wrapper(
            certificates.delete,
        )
        self.activate = to_streamed_response_wrapper(
            certificates.activate,
        )
        self.deactivate = to_streamed_response_wrapper(
            certificates.deactivate,
        )
        self.get = to_streamed_response_wrapper(
            certificates.get,
        )


class AsyncCertificatesResourceWithStreamingResponse:
    def __init__(self, certificates: AsyncCertificatesResource) -> None:
        self._certificates = certificates

        self.create = async_to_streamed_response_wrapper(
            certificates.create,
        )
        self.list = async_to_streamed_response_wrapper(
            certificates.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            certificates.delete,
        )
        self.activate = async_to_streamed_response_wrapper(
            certificates.activate,
        )
        self.deactivate = async_to_streamed_response_wrapper(
            certificates.deactivate,
        )
        self.get = async_to_streamed_response_wrapper(
            certificates.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "AnnotationListResponse",
    "Annotation",
    "AnnotationASNsDetail",
    "AnnotationASNsDetailLocations",
    "AnnotationLocationsDetail",
    "AnnotationOriginsDetail",
    "AnnotationOutage",
]


class AnnotationASNsDetailLocations(BaseModel):
    code: str

    name: str


class AnnotationASNsDetail(BaseModel):
    asn: str

    name: str

    locations: Optional[AnnotationASNsDetailLocations] = None


class AnnotationLocationsDetail(BaseModel):
    code: str

    name: str


class AnnotationOriginsDetail(BaseModel):
    name: str

    origin: str


class AnnotationOutage(BaseModel):
    outage_cause: str = FieldInfo(alias="outageCause")

    outage_type: str = FieldInfo(alias="outageType")


class Annotation(BaseModel):
    id: str

    asns: List[int]

    asns_details: List[AnnotationASNsDetail] = FieldInfo(alias="asnsDetails")

    data_source: str = FieldInfo(alias="dataSource")

    event_type: str = FieldInfo(alias="eventType")

    locations: List[str]

    locations_details: List[AnnotationLocationsDetail] = FieldInfo(alias="locationsDetails")

    origins: List[str]

    origins_details: List[AnnotationOriginsDetail] = FieldInfo(alias="originsDetails")

    outage: AnnotationOutage

    start_date: str = FieldInfo(alias="startDate")

    description: Optional[str] = None

    end_date: Optional[str] = FieldInfo(alias="endDate", default=None)

    linked_url: Optional[str] = FieldInfo(alias="linkedUrl", default=None)

    scope: Optional[str] = None


class AnnotationListResponse(BaseModel):
    annotations: List[Annotation]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["CategoryListResponse", "CategoryListResponseItem"]


class CategoryListResponseItem(BaseModel):
    kill_chain: float = FieldInfo(alias="killChain")

    name: str

    uuid: str

    mitre_attack: Optional[List[str]] = FieldInfo(alias="mitreAttack", default=None)

    mitre_capec: Optional[List[str]] = FieldInfo(alias="mitreCapec", default=None)

    shortname: Optional[str] = None


CategoryListResponse: TypeAlias = List[CategoryListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "PipelineUpdateResponse",
    "Destination",
    "DestinationBatch",
    "DestinationCompression",
    "DestinationPath",
    "Source",
    "SourceCloudflarePipelinesWorkersPipelinesHTTPSource",
    "SourceCloudflarePipelinesWorkersPipelinesHTTPSourceCORS",
    "SourceCloudflarePipelinesWorkersPipelinesBindingSource",
]


class DestinationBatch(BaseModel):
    max_bytes: int
    """Specifies rough maximum size of files."""

    max_duration_s: float
    """Specifies duration to wait to aggregate batches files."""

    max_rows: int
    """Specifies rough maximum number of rows per file."""


class DestinationCompression(BaseModel):
    type: Literal["none", "gzip", "deflate"]
    """Specifies the desired compression algorithm and format."""


class DestinationPath(BaseModel):
    bucket: str
    """Specifies the R2 Bucket to store files."""

    filename: Optional[str] = None
    """Specifies the name pattern to for individual data files."""

    filepath: Optional[str] = None
    """Specifies the name pattern for directory."""

    prefix: Optional[str] = None
    """Specifies the base directory within the bucket."""


class Destination(BaseModel):
    batch: DestinationBatch

    compression: DestinationCompression

    format: Literal["json"]
    """Specifies the format of data to deliver."""

    path: DestinationPath

    type: Literal["r2"]
    """Specifies the type of destination."""


class SourceCloudflarePipelinesWorkersPipelinesHTTPSourceCORS(BaseModel):
    origins: Optional[List[str]] = None
    """Specifies allowed origins to allow Cross Origin HTTP Requests."""


class SourceCloudflarePipelinesWorkersPipelinesHTTPSource(BaseModel):
    """[DEPRECATED] HTTP source configuration. Use the new streams API instead."""

    format: Literal["json"]
    """Specifies the format of source data."""

    type: str

    authentication: Optional[bool] = None
    """Specifies whether authentication is required to send to this pipeline via HTTP."""

    cors: Optional[SourceCloudflarePipelinesWorkersPipelinesHTTPSourceCORS] = None


class SourceCloudflarePipelinesWorkersPipelinesBindingSource(BaseModel):
    """[DEPRECATED] Worker binding source configuration.

    Use the new streams API instead.
    """

    format: Literal["json"]
    """Specifies the format of source data."""

    type: str


Source: TypeAlias = Union[
    SourceCloudflarePipelinesWorkersPipelinesHTTPSource, SourceCloudflarePipelinesWorkersPipelinesBindingSource
]


class PipelineUpdateResponse(BaseModel):
    """[DEPRECATED] Describes the configuration of a pipeline.

    Use the new streams/sinks/pipelines API instead.
    """

    id: str
    """Specifies the pipeline identifier."""

    destination: Destination

    endpoint: str
    """Indicates the endpoint URL to send traffic."""

    name: str
    """Defines the name of the pipeline."""

    source: List[Source]

    version: float
    """Indicates the version number of last saved configuration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .ttl import TTL
from ..._utils import PropertyInfo
from .a_record import ARecord
from ..._models import BaseModel
from .ds_record import DSRecord
from .mx_record import MXRecord
from .ns_record import NSRecord
from .caa_record import CAARecord
from .loc_record import LOCRecord
from .ptr_record import PTRRecord
from .srv_record import SRVRecord
from .txt_record import TXTRecord
from .uri_record import URIRecord
from .aaaa_record import AAAARecord
from .cert_record import CERTRecord
from .record_tags import RecordTags
from .svcb_record import SVCBRecord
from .tlsa_record import TLSARecord
from .cname_record import CNAMERecord
from .https_record import HTTPSRecord
from .naptr_record import NAPTRRecord
from .sshfp_record import SSHFPRecord
from .dnskey_record import DNSKEYRecord
from .smimea_record import SMIMEARecord

__all__ = [
    "RecordResponse",
    "A",
    "AAAA",
    "CNAME",
    "MX",
    "NS",
    "Openpgpkey",
    "OpenpgpkeySettings",
    "PTR",
    "TXT",
    "CAA",
    "CERT",
    "DNSKEY",
    "DS",
    "HTTPS",
    "LOC",
    "NAPTR",
    "SMIMEA",
    "SRV",
    "SSHFP",
    "SVCB",
    "TLSA",
    "URI",
]


class A(ARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class AAAA(AAAARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class CNAME(CNAMERecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class MX(MXRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class NS(NSRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class OpenpgpkeySettings(BaseModel):
    """Settings for the DNS record."""

    ipv4_only: Optional[bool] = None
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: Optional[bool] = None
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class Openpgpkey(BaseModel):
    id: str
    """Identifier."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """A single Base64-encoded OpenPGP Transferable Public Key (RFC 4880 Section 11.1)"""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    name: str
    """Complete DNS record name, including the zone name, in Punycode."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: OpenpgpkeySettings
    """Settings for the DNS record."""

    tags: List[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""

    ttl: TTL
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Literal["OPENPGPKEY"]
    """Record type."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class PTR(PTRRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class TXT(TXTRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class CAA(CAARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class CERT(CERTRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class DNSKEY(DNSKEYRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class DS(DSRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class HTTPS(HTTPSRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class LOC(LOCRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class NAPTR(NAPTRRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class SMIMEA(SMIMEARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class SRV(SRVRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class SSHFP(SSHFPRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class SVCB(SVCBRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class TLSA(TLSARecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


class URI(URIRecord):
    id: str
    """Identifier."""

    created_on: datetime
    """When the record was created."""

    meta: object
    """Extra Cloudflare-specific information about the record."""

    modified_on: datetime
    """When the record was last modified."""

    proxiable: bool
    """Whether the record can be proxied by Cloudflare or not."""

    comment_modified_on: Optional[datetime] = None
    """When the record comment was last modified. Omitted if there is no comment."""

    tags_modified_on: Optional[datetime] = None
    """When the record tags were last modified. Omitted if there are no tags."""


RecordResponse: TypeAlias = Annotated[
    Union[
        A,
        AAAA,
        CNAME,
        MX,
        NS,
        Openpgpkey,
        PTR,
        TXT,
        CAA,
        CERT,
        DNSKEY,
        DS,
        HTTPS,
        LOC,
        NAPTR,
        SMIMEA,
        SRV,
        SSHFP,
        SVCB,
        TLSA,
        URI,
    ],
    PropertyInfo(discriminator="type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["DatabaseExportResponse", "Result"]


class Result(BaseModel):
    """Only present when status = 'complete'"""

    filename: Optional[str] = None
    """The generated SQL filename."""

    signed_url: Optional[str] = None
    """The URL to download the exported SQL. Available for one hour."""


class DatabaseExportResponse(BaseModel):
    at_bookmark: Optional[str] = None
    """The current time-travel bookmark for your D1, used to poll for updates.

    Will not change for the duration of the export task.
    """

    error: Optional[str] = None
    """Only present when status = 'error'. Contains the error message."""

    messages: Optional[List[str]] = None
    """Logs since the last time you polled"""

    result: Optional[Result] = None
    """Only present when status = 'complete'"""

    status: Optional[Literal["complete", "error"]] = None

    success: Optional[bool] = None

    type: Optional[Literal["export"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TableListParams"]


class TableListParams(TypedDict, total=False):
    account_id: Required[str]
    """Use this to identify the account."""

    bucket_name: Required[str]
    """Specifies the R2 bucket name."""

    page_size: int
    """Maximum number of tables to return per page. Defaults to 100, maximum 1000."""

    page_token: str
    """
    Opaque pagination token from a previous response. Use this to fetch the next
    page of results.
    """

    return_details: bool
    """
    Whether to include additional metadata (timestamps, locations). When true,
    response includes created_at, updated_at, metadata_locations, and locations
    arrays.
    """

    return_uuids: bool
    """
    Whether to include table UUIDs in the response. Set to true to receive the
    table_uuids array.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["PayloadLogUpdateParams"]


class PayloadLogUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    masking_level: Literal["full", "partial", "clear", "default"]
    """Masking level for payload logs.

    - `full`: The entire payload is masked.
    - `partial`: Only partial payload content is masked.
    - `clear`: No masking is applied to the payload content.
    - `default`: DLP uses its default masking behavior.
    """

    public_key: Optional[str]
    """Base64-encoded public key for encrypting payload logs.

    - Set to null or empty string to disable payload logging.
    - Set to a non-empty base64 string to enable payload logging with the given key.

    For customers with configurable payload masking feature rolled out:

    - If the field is missing, the existing setting will be kept. Note that this is
      different from setting to null or empty string.

    For all other customers:

    - If the field is missing, the existing setting will be cleared.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["SecretCreateResponse"]


class SecretCreateResponse(BaseModel):
    id: str
    """Secret identifier tag."""

    created: datetime
    """Whenthe secret was created."""

    modified: datetime
    """When the secret was modified."""

    name: str
    """The name of the secret"""

    status: Literal["pending", "active", "deleted"]

    store_id: str
    """Store Identifier"""

    comment: Optional[str] = None
    """Freeform text describing the secret"""

    scopes: Optional[List[str]] = None
    """The list of services that can use this secret."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["MonitorGroupUpdateParams", "Member"]


class MonitorGroupUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    description: Required[str]
    """A short description of the monitor group"""

    members: Required[Iterable[Member]]
    """List of monitors in this group"""


class Member(TypedDict, total=False):
    enabled: Required[bool]
    """Whether this monitor is enabled in the group"""

    monitor_id: Required[str]
    """
    The ID of the Monitor to use for checking the health of origins within this
    pool.
    """

    monitoring_only: Required[bool]
    """Whether this monitor is used for monitoring only (does not affect pool health)"""

    must_be_healthy: Required[bool]
    """Whether this monitor must be healthy for the pool to be considered healthy"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["SessionListResponse", "SessionListResponseItem"]


class SessionListResponseItem(BaseModel):
    session_id: str = FieldInfo(alias="sessionId")
    """Session ID."""

    close_reason: Optional[str] = FieldInfo(alias="closeReason", default=None)
    """Reason for session closure."""

    close_reason_text: Optional[str] = FieldInfo(alias="closeReasonText", default=None)
    """Human-readable close reason."""

    connection_end_time: Optional[float] = FieldInfo(alias="connectionEndTime", default=None)
    """Connection end time."""

    connection_id: Optional[str] = FieldInfo(alias="connectionId", default=None)
    """Connection ID."""

    connection_start_time: Optional[float] = FieldInfo(alias="connectionStartTime", default=None)
    """Connection start time."""

    devtools_frontend_url: Optional[str] = FieldInfo(alias="devtoolsFrontendUrl", default=None)
    """DevTools frontend URL."""

    end_time: Optional[float] = FieldInfo(alias="endTime", default=None)
    """Session end time."""

    last_updated: Optional[float] = FieldInfo(alias="lastUpdated", default=None)
    """Last updated timestamp."""

    start_time: Optional[float] = FieldInfo(alias="startTime", default=None)
    """Session start time."""

    web_socket_debugger_url: Optional[str] = FieldInfo(alias="webSocketDebuggerUrl", default=None)
    """WebSocket URL for debugging this target."""


SessionListResponse: TypeAlias = List[SessionListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .scim import (
    SCIMResource,
    AsyncSCIMResource,
    SCIMResourceWithRawResponse,
    AsyncSCIMResourceWithRawResponse,
    SCIMResourceWithStreamingResponse,
    AsyncSCIMResourceWithStreamingResponse,
)
from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .groups import (
    GroupsResource,
    AsyncGroupsResource,
    GroupsResourceWithRawResponse,
    AsyncGroupsResourceWithRawResponse,
    GroupsResourceWithStreamingResponse,
    AsyncGroupsResourceWithStreamingResponse,
)

__all__ = [
    "GroupsResource",
    "AsyncGroupsResource",
    "GroupsResourceWithRawResponse",
    "AsyncGroupsResourceWithRawResponse",
    "GroupsResourceWithStreamingResponse",
    "AsyncGroupsResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "SCIMResource",
    "AsyncSCIMResource",
    "SCIMResourceWithRawResponse",
    "AsyncSCIMResourceWithRawResponse",
    "SCIMResourceWithStreamingResponse",
    "AsyncSCIMResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["IPGetParams"]


class IPGetParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    ipv4: str

    ipv6: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PreviewCreateParams"]


class PreviewCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    postfix_id: Required[str]
    """The identifier of the message"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["GenericOAuthConfigParam"]


class GenericOAuthConfigParam(TypedDict, total=False):
    client_id: str
    """Your OAuth Client ID"""

    client_secret: str
    """Your OAuth Client Secret"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ....._types import SequenceNotStr

__all__ = ["UserListParams"]


class UserListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    cf_resource_id: SequenceNotStr[str]
    """
    The unique Cloudflare-generated Id of the SCIM User resource; also known as the
    "Id". Pass once for a single lookup (`?cf_resource_id=A`) or repeat the
    parameter (`?cf_resource_id=A&cf_resource_id=B`) to look up multiple users in
    one request, up to 50 values. Mutually exclusive with `idp_resource_id`,
    `username`, `email`, `name`, `search_contains`, and `search_starts_with`.
    """

    email: str
    """The email address of the SCIM User resource."""

    idp_resource_id: SequenceNotStr[str]
    """
    The IdP-generated Id of the SCIM User resource; also known as the "external Id".
    Pass once for a single lookup (`?idp_resource_id=A`) or repeat the parameter
    (`?idp_resource_id=A&idp_resource_id=B`) to look up multiple users in one
    request, up to 50 values. Mutually exclusive with `cf_resource_id`, `username`,
    `email`, `name`, `search_contains`, and `search_starts_with`.
    """

    name: str
    """The name of the SCIM User resource."""

    page: int
    """Page number of results."""

    per_page: int
    """Number of results per page."""

    username: str
    """The username of the SCIM User resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .dcv_method import DCVMethod as DCVMethod
from .bundle_method import BundleMethod as BundleMethod
from .domain_validation_type import DomainValidationType as DomainValidationType
from .custom_hostname_edit_params import CustomHostnameEditParams as CustomHostnameEditParams
from .custom_hostname_list_params import CustomHostnameListParams as CustomHostnameListParams
from .custom_hostname_get_response import CustomHostnameGetResponse as CustomHostnameGetResponse
from .fallback_origin_get_response import FallbackOriginGetResponse as FallbackOriginGetResponse
from .custom_hostname_create_params import CustomHostnameCreateParams as CustomHostnameCreateParams
from .custom_hostname_edit_response import CustomHostnameEditResponse as CustomHostnameEditResponse
from .custom_hostname_list_response import CustomHostnameListResponse as CustomHostnameListResponse
from .fallback_origin_update_params import FallbackOriginUpdateParams as FallbackOriginUpdateParams
from .custom_hostname_create_response import CustomHostnameCreateResponse as CustomHostnameCreateResponse
from .custom_hostname_delete_response import CustomHostnameDeleteResponse as CustomHostnameDeleteResponse
from .fallback_origin_delete_response import FallbackOriginDeleteResponse as FallbackOriginDeleteResponse
from .fallback_origin_update_response import FallbackOriginUpdateResponse as FallbackOriginUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .user import (
    UserResource,
    AsyncUserResource,
    UserResourceWithRawResponse,
    AsyncUserResourceWithRawResponse,
    UserResourceWithStreamingResponse,
    AsyncUserResourceWithStreamingResponse,
)
from .tokens import (
    TokensResource,
    AsyncTokensResource,
    TokensResourceWithRawResponse,
    AsyncTokensResourceWithRawResponse,
    TokensResourceWithStreamingResponse,
    AsyncTokensResourceWithStreamingResponse,
)
from .billing import (
    BillingResource,
    AsyncBillingResource,
    BillingResourceWithRawResponse,
    AsyncBillingResourceWithRawResponse,
    BillingResourceWithStreamingResponse,
    AsyncBillingResourceWithStreamingResponse,
)
from .invites import (
    InvitesResource,
    AsyncInvitesResource,
    InvitesResourceWithRawResponse,
    AsyncInvitesResourceWithRawResponse,
    InvitesResourceWithStreamingResponse,
    AsyncInvitesResourceWithStreamingResponse,
)
from .audit_logs import (
    AuditLogsResource,
    AsyncAuditLogsResource,
    AuditLogsResourceWithRawResponse,
    AsyncAuditLogsResourceWithRawResponse,
    AuditLogsResourceWithStreamingResponse,
    AsyncAuditLogsResourceWithStreamingResponse,
)
from .organizations import (
    OrganizationsResource,
    AsyncOrganizationsResource,
    OrganizationsResourceWithRawResponse,
    AsyncOrganizationsResourceWithRawResponse,
    OrganizationsResourceWithStreamingResponse,
    AsyncOrganizationsResourceWithStreamingResponse,
)
from .subscriptions import (
    SubscriptionsResource,
    AsyncSubscriptionsResource,
    SubscriptionsResourceWithRawResponse,
    AsyncSubscriptionsResourceWithRawResponse,
    SubscriptionsResourceWithStreamingResponse,
    AsyncSubscriptionsResourceWithStreamingResponse,
)

__all__ = [
    "AuditLogsResource",
    "AsyncAuditLogsResource",
    "AuditLogsResourceWithRawResponse",
    "AsyncAuditLogsResourceWithRawResponse",
    "AuditLogsResourceWithStreamingResponse",
    "AsyncAuditLogsResourceWithStreamingResponse",
    "BillingResource",
    "AsyncBillingResource",
    "BillingResourceWithRawResponse",
    "AsyncBillingResourceWithRawResponse",
    "BillingResourceWithStreamingResponse",
    "AsyncBillingResourceWithStreamingResponse",
    "InvitesResource",
    "AsyncInvitesResource",
    "InvitesResourceWithRawResponse",
    "AsyncInvitesResourceWithRawResponse",
    "InvitesResourceWithStreamingResponse",
    "AsyncInvitesResourceWithStreamingResponse",
    "OrganizationsResource",
    "AsyncOrganizationsResource",
    "OrganizationsResourceWithRawResponse",
    "AsyncOrganizationsResourceWithRawResponse",
    "OrganizationsResourceWithStreamingResponse",
    "AsyncOrganizationsResourceWithStreamingResponse",
    "SubscriptionsResource",
    "AsyncSubscriptionsResource",
    "SubscriptionsResourceWithRawResponse",
    "AsyncSubscriptionsResourceWithRawResponse",
    "SubscriptionsResourceWithStreamingResponse",
    "AsyncSubscriptionsResourceWithStreamingResponse",
    "TokensResource",
    "AsyncTokensResource",
    "TokensResourceWithRawResponse",
    "AsyncTokensResourceWithRawResponse",
    "TokensResourceWithStreamingResponse",
    "AsyncTokensResourceWithStreamingResponse",
    "UserResource",
    "AsyncUserResource",
    "UserResourceWithRawResponse",
    "AsyncUserResourceWithRawResponse",
    "UserResourceWithStreamingResponse",
    "AsyncUserResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ResponseBuffering"]


class ResponseBuffering(BaseModel):
    id: Optional[Literal["response_buffering"]] = None
    """
    Turn on or off whether Cloudflare should wait for an entire file from the origin
    server before forwarding it to the site visitor. By default, Cloudflare sends
    packets to the client as they arrive from the origin server.
    """

    value: Optional[Literal["on", "off"]] = None
    """The status of Response Buffering"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.addressing.address_maps import ip_update_params
from ....types.addressing.address_maps.ip_delete_response import IPDeleteResponse
from ....types.addressing.address_maps.ip_update_response import IPUpdateResponse

__all__ = ["IPsResource", "AsyncIPsResource"]


class IPsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IPsResourceWithStreamingResponse(self)

    def update(
        self,
        ip_address: str,
        *,
        account_id: str,
        address_map_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPUpdateResponse:
        """
        Add an IP from a prefix owned by the account to a particular address map.

        Args:
          account_id: Identifier of a Cloudflare account.

          address_map_id: Identifier of an Address Map.

          ip_address: An IPv4 or IPv6 address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not address_map_id:
            raise ValueError(f"Expected a non-empty value for `address_map_id` but received {address_map_id!r}")
        if not ip_address:
            raise ValueError(f"Expected a non-empty value for `ip_address` but received {ip_address!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/addressing/address_maps/{address_map_id}/ips/{ip_address}",
                account_id=account_id,
                address_map_id=address_map_id,
                ip_address=ip_address,
            ),
            body=maybe_transform(body, ip_update_params.IPUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPUpdateResponse,
        )

    def delete(
        self,
        ip_address: str,
        *,
        account_id: str,
        address_map_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPDeleteResponse:
        """
        Remove an IP from a particular address map.

        Args:
          account_id: Identifier of a Cloudflare account.

          address_map_id: Identifier of an Address Map.

          ip_address: An IPv4 or IPv6 address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not address_map_id:
            raise ValueError(f"Expected a non-empty value for `address_map_id` but received {address_map_id!r}")
        if not ip_address:
            raise ValueError(f"Expected a non-empty value for `ip_address` but received {ip_address!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/addressing/address_maps/{address_map_id}/ips/{ip_address}",
                account_id=account_id,
                address_map_id=address_map_id,
                ip_address=ip_address,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPDeleteResponse,
        )


class AsyncIPsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIPsResourceWithStreamingResponse(self)

    async def update(
        self,
        ip_address: str,
        *,
        account_id: str,
        address_map_id: str,
        body: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPUpdateResponse:
        """
        Add an IP from a prefix owned by the account to a particular address map.

        Args:
          account_id: Identifier of a Cloudflare account.

          address_map_id: Identifier of an Address Map.

          ip_address: An IPv4 or IPv6 address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not address_map_id:
            raise ValueError(f"Expected a non-empty value for `address_map_id` but received {address_map_id!r}")
        if not ip_address:
            raise ValueError(f"Expected a non-empty value for `ip_address` but received {ip_address!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/addressing/address_maps/{address_map_id}/ips/{ip_address}",
                account_id=account_id,
                address_map_id=address_map_id,
                ip_address=ip_address,
            ),
            body=await async_maybe_transform(body, ip_update_params.IPUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPUpdateResponse,
        )

    async def delete(
        self,
        ip_address: str,
        *,
        account_id: str,
        address_map_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IPDeleteResponse:
        """
        Remove an IP from a particular address map.

        Args:
          account_id: Identifier of a Cloudflare account.

          address_map_id: Identifier of an Address Map.

          ip_address: An IPv4 or IPv6 address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not address_map_id:
            raise ValueError(f"Expected a non-empty value for `address_map_id` but received {address_map_id!r}")
        if not ip_address:
            raise ValueError(f"Expected a non-empty value for `ip_address` but received {ip_address!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/addressing/address_maps/{address_map_id}/ips/{ip_address}",
                account_id=account_id,
                address_map_id=address_map_id,
                ip_address=ip_address,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IPDeleteResponse,
        )


class IPsResourceWithRawResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.update = to_raw_response_wrapper(
            ips.update,
        )
        self.delete = to_raw_response_wrapper(
            ips.delete,
        )


class AsyncIPsResourceWithRawResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.update = async_to_raw_response_wrapper(
            ips.update,
        )
        self.delete = async_to_raw_response_wrapper(
            ips.delete,
        )


class IPsResourceWithStreamingResponse:
    def __init__(self, ips: IPsResource) -> None:
        self._ips = ips

        self.update = to_streamed_response_wrapper(
            ips.update,
        )
        self.delete = to_streamed_response_wrapper(
            ips.delete,
        )


class AsyncIPsResourceWithStreamingResponse:
    def __init__(self, ips: AsyncIPsResource) -> None:
        self._ips = ips

        self.update = async_to_streamed_response_wrapper(
            ips.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            ips.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["RequestValidity"]

RequestValidity: TypeAlias = Literal[7, 30, 90, 365, 730, 1095, 5475]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["LogoDeleteResponse"]


class LogoDeleteResponse(BaseModel):
    message: str

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["CustomDeleteResponse"]


class CustomDeleteResponse(BaseModel):
    domain: str
    """Name of the removed custom domain."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["SAMLGroupRule", "SAML"]


class SAML(BaseModel):
    attribute_name: str
    """The name of the SAML attribute."""

    attribute_value: str
    """The SAML attribute value to look for."""

    identity_provider_id: str
    """The ID of your SAML identity provider."""


class SAMLGroupRule(BaseModel):
    """
    Matches a SAML group.
    Requires a SAML identity provider.
    """

    saml: SAML



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RouteCreateParams"]


class RouteCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account ID"""

    network: Required[str]
    """The private IPv4 or IPv6 range connected by the route, in CIDR notation."""

    tunnel_id: Required[str]
    """UUID of the tunnel."""

    comment: str
    """Optional remark describing the route."""

    virtual_network_id: str
    """UUID of the virtual network."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .target_get_response import TargetGetResponse as TargetGetResponse
from .target_create_params import TargetCreateParams as TargetCreateParams
from .target_list_response import TargetListResponse as TargetListResponse
from .target_close_response import TargetCloseResponse as TargetCloseResponse
from .target_create_response import TargetCreateResponse as TargetCreateResponse
from .target_activate_response import TargetActivateResponse as TargetActivateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RequestCreateParams"]


class RequestCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    content: str
    """Request content."""

    priority: str
    """Priority for analyzing the request."""

    request_type: str
    """Requested information from request."""

    summary: str
    """Brief description of the request."""

    tlp: Literal["clear", "amber", "amber-strict", "green", "red"]
    """The CISA defined Traffic Light Protocol (TLP)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .healthcheck import Healthcheck as Healthcheck
from .check_region import CheckRegion as CheckRegion
from .tcp_configuration import TCPConfiguration as TCPConfiguration
from .http_configuration import HTTPConfiguration as HTTPConfiguration
from .preview_create_params import PreviewCreateParams as PreviewCreateParams
from .healthcheck_edit_params import HealthcheckEditParams as HealthcheckEditParams
from .healthcheck_list_params import HealthcheckListParams as HealthcheckListParams
from .preview_delete_response import PreviewDeleteResponse as PreviewDeleteResponse
from .tcp_configuration_param import TCPConfigurationParam as TCPConfigurationParam
from .http_configuration_param import HTTPConfigurationParam as HTTPConfigurationParam
from .healthcheck_create_params import HealthcheckCreateParams as HealthcheckCreateParams
from .healthcheck_update_params import HealthcheckUpdateParams as HealthcheckUpdateParams
from .healthcheck_delete_response import HealthcheckDeleteResponse as HealthcheckDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = [
    "PreviewGetResponse",
    "PreviewGetResponseItem",
    "PreviewGetResponseItemOriginPreviewGetResponseItemOriginItem",
]


class PreviewGetResponseItemOriginPreviewGetResponseItemOriginItem(BaseModel):
    """The origin ipv4/ipv6 address or domain name mapped to its health data."""

    failure_reason: Optional[str] = None

    healthy: Optional[bool] = None

    response_code: Optional[float] = None

    rtt: Optional[str] = None


class PreviewGetResponseItem(BaseModel):
    healthy: Optional[bool] = None

    origins: Optional[List[Dict[str, PreviewGetResponseItemOriginPreviewGetResponseItemOriginItem]]] = None


PreviewGetResponse: TypeAlias = Dict[str, PreviewGetResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["MinTLSVersion"]


class MinTLSVersion(BaseModel):
    """Only accepts HTTPS requests that use at least the TLS protocol version specified.

    For example, if TLS 1.1 is selected, TLS 1.0 connections will be rejected, while 1.1, 1.2, and 1.3 (if enabled) will be permitted.
    """

    id: Literal["min_tls_version"]
    """ID of the zone setting."""

    value: Literal["1.0", "1.1", "1.2", "1.3"]
    """Current value of the zone setting."""

    editable: Optional[Literal[True, False]] = None
    """
    Whether or not this setting can be modified for this zone (based on your
    Cloudflare plan level).
    """

    modified_on: Optional[datetime] = None
    """last time this setting was modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.zones import environment_edit_params, environment_create_params, environment_update_params
from ..._base_client import make_request_options
from ...types.zones.environment_edit_response import EnvironmentEditResponse
from ...types.zones.environment_list_response import EnvironmentListResponse
from ...types.zones.environment_create_response import EnvironmentCreateResponse
from ...types.zones.environment_delete_response import EnvironmentDeleteResponse
from ...types.zones.environment_update_response import EnvironmentUpdateResponse
from ...types.zones.environment_rollback_response import EnvironmentRollbackResponse

__all__ = ["EnvironmentsResource", "AsyncEnvironmentsResource"]


class EnvironmentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EnvironmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EnvironmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EnvironmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EnvironmentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        environments: Iterable[environment_create_params.Environment],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentCreateResponse:
        """
        Create zone environments

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/environments", zone_id=zone_id),
            body=maybe_transform({"environments": environments}, environment_create_params.EnvironmentCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentCreateResponse], ResultWrapper[EnvironmentCreateResponse]),
        )

    def update(
        self,
        *,
        zone_id: str,
        environments: Iterable[environment_update_params.Environment],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentUpdateResponse:
        """
        Upsert zone environments

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._put(
            path_template("/zones/{zone_id}/environments", zone_id=zone_id),
            body=maybe_transform({"environments": environments}, environment_update_params.EnvironmentUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentUpdateResponse], ResultWrapper[EnvironmentUpdateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentListResponse:
        """
        List zone environments

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/environments", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentListResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentListResponse], ResultWrapper[EnvironmentListResponse]),
        )

    def delete(
        self,
        environment_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentDeleteResponse:
        """
        Delete zone environment

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not environment_id:
            raise ValueError(f"Expected a non-empty value for `environment_id` but received {environment_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/environments/{environment_id}", zone_id=zone_id, environment_id=environment_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentDeleteResponse], ResultWrapper[EnvironmentDeleteResponse]),
        )

    def edit(
        self,
        *,
        zone_id: str,
        environments: Iterable[environment_edit_params.Environment],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentEditResponse:
        """
        Partially update zone environments

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._patch(
            path_template("/zones/{zone_id}/environments", zone_id=zone_id),
            body=maybe_transform({"environments": environments}, environment_edit_params.EnvironmentEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentEditResponse], ResultWrapper[EnvironmentEditResponse]),
        )

    def rollback(
        self,
        environment_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentRollbackResponse:
        """
        Roll back zone environment

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not environment_id:
            raise ValueError(f"Expected a non-empty value for `environment_id` but received {environment_id!r}")
        return self._post(
            path_template(
                "/zones/{zone_id}/environments/{environment_id}/rollback",
                zone_id=zone_id,
                environment_id=environment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentRollbackResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentRollbackResponse], ResultWrapper[EnvironmentRollbackResponse]),
        )


class AsyncEnvironmentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEnvironmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEnvironmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEnvironmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEnvironmentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        environments: Iterable[environment_create_params.Environment],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentCreateResponse:
        """
        Create zone environments

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/environments", zone_id=zone_id),
            body=await async_maybe_transform(
                {"environments": environments}, environment_create_params.EnvironmentCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentCreateResponse], ResultWrapper[EnvironmentCreateResponse]),
        )

    async def update(
        self,
        *,
        zone_id: str,
        environments: Iterable[environment_update_params.Environment],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentUpdateResponse:
        """
        Upsert zone environments

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._put(
            path_template("/zones/{zone_id}/environments", zone_id=zone_id),
            body=await async_maybe_transform(
                {"environments": environments}, environment_update_params.EnvironmentUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentUpdateResponse], ResultWrapper[EnvironmentUpdateResponse]),
        )

    async def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentListResponse:
        """
        List zone environments

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/environments", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentListResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentListResponse], ResultWrapper[EnvironmentListResponse]),
        )

    async def delete(
        self,
        environment_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentDeleteResponse:
        """
        Delete zone environment

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not environment_id:
            raise ValueError(f"Expected a non-empty value for `environment_id` but received {environment_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/environments/{environment_id}", zone_id=zone_id, environment_id=environment_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentDeleteResponse], ResultWrapper[EnvironmentDeleteResponse]),
        )

    async def edit(
        self,
        *,
        zone_id: str,
        environments: Iterable[environment_edit_params.Environment],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentEditResponse:
        """
        Partially update zone environments

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._patch(
            path_template("/zones/{zone_id}/environments", zone_id=zone_id),
            body=await async_maybe_transform(
                {"environments": environments}, environment_edit_params.EnvironmentEditParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentEditResponse], ResultWrapper[EnvironmentEditResponse]),
        )

    async def rollback(
        self,
        environment_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EnvironmentRollbackResponse:
        """
        Roll back zone environment

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not environment_id:
            raise ValueError(f"Expected a non-empty value for `environment_id` but received {environment_id!r}")
        return await self._post(
            path_template(
                "/zones/{zone_id}/environments/{environment_id}/rollback",
                zone_id=zone_id,
                environment_id=environment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EnvironmentRollbackResponse]._unwrapper,
            ),
            cast_to=cast(Type[EnvironmentRollbackResponse], ResultWrapper[EnvironmentRollbackResponse]),
        )


class EnvironmentsResourceWithRawResponse:
    def __init__(self, environments: EnvironmentsResource) -> None:
        self._environments = environments

        self.create = to_raw_response_wrapper(
            environments.create,
        )
        self.update = to_raw_response_wrapper(
            environments.update,
        )
        self.list = to_raw_response_wrapper(
            environments.list,
        )
        self.delete = to_raw_response_wrapper(
            environments.delete,
        )
        self.edit = to_raw_response_wrapper(
            environments.edit,
        )
        self.rollback = to_raw_response_wrapper(
            environments.rollback,
        )


class AsyncEnvironmentsResourceWithRawResponse:
    def __init__(self, environments: AsyncEnvironmentsResource) -> None:
        self._environments = environments

        self.create = async_to_raw_response_wrapper(
            environments.create,
        )
        self.update = async_to_raw_response_wrapper(
            environments.update,
        )
        self.list = async_to_raw_response_wrapper(
            environments.list,
        )
        self.delete = async_to_raw_response_wrapper(
            environments.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            environments.edit,
        )
        self.rollback = async_to_raw_response_wrapper(
            environments.rollback,
        )


class EnvironmentsResourceWithStreamingResponse:
    def __init__(self, environments: EnvironmentsResource) -> None:
        self._environments = environments

        self.create = to_streamed_response_wrapper(
            environments.create,
        )
        self.update = to_streamed_response_wrapper(
            environments.update,
        )
        self.list = to_streamed_response_wrapper(
            environments.list,
        )
        self.delete = to_streamed_response_wrapper(
            environments.delete,
        )
        self.edit = to_streamed_response_wrapper(
            environments.edit,
        )
        self.rollback = to_streamed_response_wrapper(
            environments.rollback,
        )


class AsyncEnvironmentsResourceWithStreamingResponse:
    def __init__(self, environments: AsyncEnvironmentsResource) -> None:
        self._environments = environments

        self.create = async_to_streamed_response_wrapper(
            environments.create,
        )
        self.update = async_to_streamed_response_wrapper(
            environments.update,
        )
        self.list = async_to_streamed_response_wrapper(
            environments.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            environments.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            environments.edit,
        )
        self.rollback = async_to_streamed_response_wrapper(
            environments.rollback,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .v1.v1 import (
    V1Resource,
    AsyncV1Resource,
    V1ResourceWithRawResponse,
    AsyncV1ResourceWithRawResponse,
    V1ResourceWithStreamingResponse,
    AsyncV1ResourceWithStreamingResponse,
)
from .v2.v2 import (
    V2Resource,
    AsyncV2Resource,
    V2ResourceWithRawResponse,
    AsyncV2ResourceWithRawResponse,
    V2ResourceWithStreamingResponse,
    AsyncV2ResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ImagesResource", "AsyncImagesResource"]


class ImagesResource(SyncAPIResource):
    @cached_property
    def v1(self) -> V1Resource:
        return V1Resource(self._client)

    @cached_property
    def v2(self) -> V2Resource:
        return V2Resource(self._client)

    @cached_property
    def with_raw_response(self) -> ImagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ImagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ImagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ImagesResourceWithStreamingResponse(self)


class AsyncImagesResource(AsyncAPIResource):
    @cached_property
    def v1(self) -> AsyncV1Resource:
        return AsyncV1Resource(self._client)

    @cached_property
    def v2(self) -> AsyncV2Resource:
        return AsyncV2Resource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncImagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncImagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncImagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncImagesResourceWithStreamingResponse(self)


class ImagesResourceWithRawResponse:
    def __init__(self, images: ImagesResource) -> None:
        self._images = images

    @cached_property
    def v1(self) -> V1ResourceWithRawResponse:
        return V1ResourceWithRawResponse(self._images.v1)

    @cached_property
    def v2(self) -> V2ResourceWithRawResponse:
        return V2ResourceWithRawResponse(self._images.v2)


class AsyncImagesResourceWithRawResponse:
    def __init__(self, images: AsyncImagesResource) -> None:
        self._images = images

    @cached_property
    def v1(self) -> AsyncV1ResourceWithRawResponse:
        return AsyncV1ResourceWithRawResponse(self._images.v1)

    @cached_property
    def v2(self) -> AsyncV2ResourceWithRawResponse:
        return AsyncV2ResourceWithRawResponse(self._images.v2)


class ImagesResourceWithStreamingResponse:
    def __init__(self, images: ImagesResource) -> None:
        self._images = images

    @cached_property
    def v1(self) -> V1ResourceWithStreamingResponse:
        return V1ResourceWithStreamingResponse(self._images.v1)

    @cached_property
    def v2(self) -> V2ResourceWithStreamingResponse:
        return V2ResourceWithStreamingResponse(self._images.v2)


class AsyncImagesResourceWithStreamingResponse:
    def __init__(self, images: AsyncImagesResource) -> None:
        self._images = images

    @cached_property
    def v1(self) -> AsyncV1ResourceWithStreamingResponse:
        return AsyncV1ResourceWithStreamingResponse(self._images.v1)

    @cached_property
    def v2(self) -> AsyncV2ResourceWithStreamingResponse:
        return AsyncV2ResourceWithStreamingResponse(self._images.v2)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.speed.availability import Availability

__all__ = ["AvailabilitiesResource", "AsyncAvailabilitiesResource"]


class AvailabilitiesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AvailabilitiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AvailabilitiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AvailabilitiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AvailabilitiesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Availability]:
        """
        Retrieves quota for all plans, as well as the current zone quota.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/speed_api/availabilities", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Availability]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Availability]], ResultWrapper[Availability]),
        )


class AsyncAvailabilitiesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAvailabilitiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAvailabilitiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAvailabilitiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAvailabilitiesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Availability]:
        """
        Retrieves quota for all plans, as well as the current zone quota.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/speed_api/availabilities", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Availability]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Availability]], ResultWrapper[Availability]),
        )


class AvailabilitiesResourceWithRawResponse:
    def __init__(self, availabilities: AvailabilitiesResource) -> None:
        self._availabilities = availabilities

        self.list = to_raw_response_wrapper(
            availabilities.list,
        )


class AsyncAvailabilitiesResourceWithRawResponse:
    def __init__(self, availabilities: AsyncAvailabilitiesResource) -> None:
        self._availabilities = availabilities

        self.list = async_to_raw_response_wrapper(
            availabilities.list,
        )


class AvailabilitiesResourceWithStreamingResponse:
    def __init__(self, availabilities: AvailabilitiesResource) -> None:
        self._availabilities = availabilities

        self.list = to_streamed_response_wrapper(
            availabilities.list,
        )


class AsyncAvailabilitiesResourceWithStreamingResponse:
    def __init__(self, availabilities: AsyncAvailabilitiesResource) -> None:
        self._availabilities = availabilities

        self.list = async_to_streamed_response_wrapper(
            availabilities.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "ProjectCreateParams",
    "BuildConfig",
    "DeploymentConfigs",
    "DeploymentConfigsPreview",
    "DeploymentConfigsPreviewAIBindings",
    "DeploymentConfigsPreviewAnalyticsEngineDatasets",
    "DeploymentConfigsPreviewBrowsers",
    "DeploymentConfigsPreviewD1Databases",
    "DeploymentConfigsPreviewDurableObjectNamespaces",
    "DeploymentConfigsPreviewEnvVars",
    "DeploymentConfigsPreviewEnvVarsPagesPlainTextEnvVar",
    "DeploymentConfigsPreviewEnvVarsPagesSecretTextEnvVar",
    "DeploymentConfigsPreviewHyperdriveBindings",
    "DeploymentConfigsPreviewKVNamespaces",
    "DeploymentConfigsPreviewLimits",
    "DeploymentConfigsPreviewMTLSCertificates",
    "DeploymentConfigsPreviewPlacement",
    "DeploymentConfigsPreviewQueueProducers",
    "DeploymentConfigsPreviewR2Buckets",
    "DeploymentConfigsPreviewServices",
    "DeploymentConfigsPreviewVectorizeBindings",
    "DeploymentConfigsProduction",
    "DeploymentConfigsProductionAIBindings",
    "DeploymentConfigsProductionAnalyticsEngineDatasets",
    "DeploymentConfigsProductionBrowsers",
    "DeploymentConfigsProductionD1Databases",
    "DeploymentConfigsProductionDurableObjectNamespaces",
    "DeploymentConfigsProductionEnvVars",
    "DeploymentConfigsProductionEnvVarsPagesPlainTextEnvVar",
    "DeploymentConfigsProductionEnvVarsPagesSecretTextEnvVar",
    "DeploymentConfigsProductionHyperdriveBindings",
    "DeploymentConfigsProductionKVNamespaces",
    "DeploymentConfigsProductionLimits",
    "DeploymentConfigsProductionMTLSCertificates",
    "DeploymentConfigsProductionPlacement",
    "DeploymentConfigsProductionQueueProducers",
    "DeploymentConfigsProductionR2Buckets",
    "DeploymentConfigsProductionServices",
    "DeploymentConfigsProductionVectorizeBindings",
    "Source",
    "SourceConfig",
]


class ProjectCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    name: Required[str]
    """Name of the project."""

    production_branch: Required[str]
    """Production branch of the project. Used to identify production deployments."""

    build_config: BuildConfig
    """Configs for the project build process."""

    deployment_configs: DeploymentConfigs
    """Configs for deployments in a project."""

    source: Source
    """Configs for the project source control."""


class BuildConfig(TypedDict, total=False):
    """Configs for the project build process."""

    build_caching: bool
    """Enable build caching for the project."""

    build_command: str
    """Command used to build project."""

    destination_dir: str
    """Output directory of the build."""

    root_dir: str
    """Directory to run the command."""

    web_analytics_tag: Optional[str]
    """The classifying tag for analytics."""

    web_analytics_token: Optional[str]
    """The auth token for analytics."""


class DeploymentConfigsPreviewAIBindings(TypedDict, total=False):
    """AI binding."""

    project_id: Required[str]


class DeploymentConfigsPreviewAnalyticsEngineDatasets(TypedDict, total=False):
    """Analytics Engine binding."""

    dataset: Required[str]
    """Name of the dataset."""


class DeploymentConfigsPreviewBrowsers(TypedDict, total=False):
    """Browser binding."""

    pass


class DeploymentConfigsPreviewD1Databases(TypedDict, total=False):
    """D1 binding."""

    id: Required[str]
    """UUID of the D1 database."""


class DeploymentConfigsPreviewDurableObjectNamespaces(TypedDict, total=False):
    """Durable Object binding."""

    namespace_id: Required[str]
    """ID of the Durable Object namespace."""


class DeploymentConfigsPreviewEnvVarsPagesPlainTextEnvVar(TypedDict, total=False):
    """A plaintext environment variable."""

    type: Required[Literal["plain_text"]]

    value: Required[str]
    """Environment variable value."""


class DeploymentConfigsPreviewEnvVarsPagesSecretTextEnvVar(TypedDict, total=False):
    """An encrypted environment variable."""

    type: Required[Literal["secret_text"]]

    value: Required[str]
    """Secret value."""


DeploymentConfigsPreviewEnvVars: TypeAlias = Union[
    Optional[DeploymentConfigsPreviewEnvVarsPagesPlainTextEnvVar],
    Optional[DeploymentConfigsPreviewEnvVarsPagesSecretTextEnvVar],
]


class DeploymentConfigsPreviewHyperdriveBindings(TypedDict, total=False):
    """Hyperdrive binding."""

    id: Required[str]


class DeploymentConfigsPreviewKVNamespaces(TypedDict, total=False):
    """KV namespace binding."""

    namespace_id: Required[str]
    """ID of the KV namespace."""


class DeploymentConfigsPreviewLimits(TypedDict, total=False):
    """Limits for Pages Functions."""

    cpu_ms: Required[int]
    """CPU time limit in milliseconds."""


class DeploymentConfigsPreviewMTLSCertificates(TypedDict, total=False):
    """mTLS binding."""

    certificate_id: Required[str]


class DeploymentConfigsPreviewPlacement(TypedDict, total=False):
    """Placement setting used for Pages Functions."""

    mode: Required[str]
    """Placement mode."""


class DeploymentConfigsPreviewQueueProducers(TypedDict, total=False):
    """Queue Producer binding."""

    name: Required[str]
    """Name of the Queue."""


class DeploymentConfigsPreviewR2Buckets(TypedDict, total=False):
    """R2 binding."""

    name: Required[str]
    """Name of the R2 bucket."""

    jurisdiction: Optional[str]
    """Jurisdiction of the R2 bucket."""


class DeploymentConfigsPreviewServices(TypedDict, total=False):
    """Service binding."""

    service: Required[str]
    """The Service name."""

    entrypoint: Optional[str]
    """The entrypoint to bind to."""

    environment: str
    """The Service environment."""


class DeploymentConfigsPreviewVectorizeBindings(TypedDict, total=False):
    """Vectorize binding."""

    index_name: Required[str]


class DeploymentConfigsPreview(TypedDict, total=False):
    """Configs for preview deploys."""

    ai_bindings: Dict[str, Optional[DeploymentConfigsPreviewAIBindings]]
    """Constellation bindings used for Pages Functions."""

    always_use_latest_compatibility_date: bool
    """Whether to always use the latest compatibility date for Pages Functions."""

    analytics_engine_datasets: Dict[str, Optional[DeploymentConfigsPreviewAnalyticsEngineDatasets]]
    """Analytics Engine bindings used for Pages Functions."""

    browsers: Dict[str, Optional[DeploymentConfigsPreviewBrowsers]]
    """Browser bindings used for Pages Functions."""

    build_image_major_version: int
    """The major version of the build image to use for Pages Functions."""

    compatibility_date: str
    """Compatibility date used for Pages Functions."""

    compatibility_flags: SequenceNotStr[str]
    """Compatibility flags used for Pages Functions."""

    d1_databases: Dict[str, Optional[DeploymentConfigsPreviewD1Databases]]
    """D1 databases used for Pages Functions."""

    durable_object_namespaces: Dict[str, Optional[DeploymentConfigsPreviewDurableObjectNamespaces]]
    """Durable Object namespaces used for Pages Functions."""

    env_vars: Dict[str, Optional[DeploymentConfigsPreviewEnvVars]]
    """Environment variables used for builds and Pages Functions."""

    fail_open: bool
    """Whether to fail open when the deployment config cannot be applied."""

    hyperdrive_bindings: Dict[str, Optional[DeploymentConfigsPreviewHyperdriveBindings]]
    """Hyperdrive bindings used for Pages Functions."""

    kv_namespaces: Dict[str, Optional[DeploymentConfigsPreviewKVNamespaces]]
    """KV namespaces used for Pages Functions."""

    limits: DeploymentConfigsPreviewLimits
    """Limits for Pages Functions."""

    mtls_certificates: Dict[str, Optional[DeploymentConfigsPreviewMTLSCertificates]]
    """mTLS bindings used for Pages Functions."""

    placement: DeploymentConfigsPreviewPlacement
    """Placement setting used for Pages Functions."""

    queue_producers: Dict[str, Optional[DeploymentConfigsPreviewQueueProducers]]
    """Queue Producer bindings used for Pages Functions."""

    r2_buckets: Dict[str, Optional[DeploymentConfigsPreviewR2Buckets]]
    """R2 buckets used for Pages Functions."""

    services: Dict[str, Optional[DeploymentConfigsPreviewServices]]
    """Services used for Pages Functions."""

    usage_model: Literal["standard", "bundled", "unbound"]
    """The usage model for Pages Functions."""

    vectorize_bindings: Dict[str, Optional[DeploymentConfigsPreviewVectorizeBindings]]
    """Vectorize bindings used for Pages Functions."""

    wrangler_config_hash: str
    """Hash of the Wrangler configuration used for the deployment."""


class DeploymentConfigsProductionAIBindings(TypedDict, total=False):
    """AI binding."""

    project_id: Required[str]


class DeploymentConfigsProductionAnalyticsEngineDatasets(TypedDict, total=False):
    """Analytics Engine binding."""

    dataset: Required[str]
    """Name of the dataset."""


class DeploymentConfigsProductionBrowsers(TypedDict, total=False):
    """Browser binding."""

    pass


class DeploymentConfigsProductionD1Databases(TypedDict, total=False):
    """D1 binding."""

    id: Required[str]
    """UUID of the D1 database."""


class DeploymentConfigsProductionDurableObjectNamespaces(TypedDict, total=False):
    """Durable Object binding."""

    namespace_id: Required[str]
    """ID of the Durable Object namespace."""


class DeploymentConfigsProductionEnvVarsPagesPlainTextEnvVar(TypedDict, total=False):
    """A plaintext environment variable."""

    type: Required[Literal["plain_text"]]

    value: Required[str]
    """Environment variable value."""


class DeploymentConfigsProductionEnvVarsPagesSecretTextEnvVar(TypedDict, total=False):
    """An encrypted environment variable."""

    type: Required[Literal["secret_text"]]

    value: Required[str]
    """Secret value."""


DeploymentConfigsProductionEnvVars: TypeAlias = Union[
    Optional[DeploymentConfigsProductionEnvVarsPagesPlainTextEnvVar],
    Optional[DeploymentConfigsProductionEnvVarsPagesSecretTextEnvVar],
]


class DeploymentConfigsProductionHyperdriveBindings(TypedDict, total=False):
    """Hyperdrive binding."""

    id: Required[str]


class DeploymentConfigsProductionKVNamespaces(TypedDict, total=False):
    """KV namespace binding."""

    namespace_id: Required[str]
    """ID of the KV namespace."""


class DeploymentConfigsProductionLimits(TypedDict, total=False):
    """Limits for Pages Functions."""

    cpu_ms: Required[int]
    """CPU time limit in milliseconds."""


class DeploymentConfigsProductionMTLSCertificates(TypedDict, total=False):
    """mTLS binding."""

    certificate_id: Required[str]


class DeploymentConfigsProductionPlacement(TypedDict, total=False):
    """Placement setting used for Pages Functions."""

    mode: Required[str]
    """Placement mode."""


class DeploymentConfigsProductionQueueProducers(TypedDict, total=False):
    """Queue Producer binding."""

    name: Required[str]
    """Name of the Queue."""


class DeploymentConfigsProductionR2Buckets(TypedDict, total=False):
    """R2 binding."""

    name: Required[str]
    """Name of the R2 bucket."""

    jurisdiction: Optional[str]
    """Jurisdiction of the R2 bucket."""


class DeploymentConfigsProductionServices(TypedDict, total=False):
    """Service binding."""

    service: Required[str]
    """The Service name."""

    entrypoint: Optional[str]
    """The entrypoint to bind to."""

    environment: str
    """The Service environment."""


class DeploymentConfigsProductionVectorizeBindings(TypedDict, total=False):
    """Vectorize binding."""

    index_name: Required[str]


class DeploymentConfigsProduction(TypedDict, total=False):
    """Configs for production deploys."""

    ai_bindings: Dict[str, Optional[DeploymentConfigsProductionAIBindings]]
    """Constellation bindings used for Pages Functions."""

    always_use_latest_compatibility_date: bool
    """Whether to always use the latest compatibility date for Pages Functions."""

    analytics_engine_datasets: Dict[str, Optional[DeploymentConfigsProductionAnalyticsEngineDatasets]]
    """Analytics Engine bindings used for Pages Functions."""

    browsers: Dict[str, Optional[DeploymentConfigsProductionBrowsers]]
    """Browser bindings used for Pages Functions."""

    build_image_major_version: int
    """The major version of the build image to use for Pages Functions."""

    compatibility_date: str
    """Compatibility date used for Pages Functions."""

    compatibility_flags: SequenceNotStr[str]
    """Compatibility flags used for Pages Functions."""

    d1_databases: Dict[str, Optional[DeploymentConfigsProductionD1Databases]]
    """D1 databases used for Pages Functions."""

    durable_object_namespaces: Dict[str, Optional[DeploymentConfigsProductionDurableObjectNamespaces]]
    """Durable Object namespaces used for Pages Functions."""

    env_vars: Dict[str, Optional[DeploymentConfigsProductionEnvVars]]
    """Environment variables used for builds and Pages Functions."""

    fail_open: bool
    """Whether to fail open when the deployment config cannot be applied."""

    hyperdrive_bindings: Dict[str, Optional[DeploymentConfigsProductionHyperdriveBindings]]
    """Hyperdrive bindings used for Pages Functions."""

    kv_namespaces: Dict[str, Optional[DeploymentConfigsProductionKVNamespaces]]
    """KV namespaces used for Pages Functions."""

    limits: DeploymentConfigsProductionLimits
    """Limits for Pages Functions."""

    mtls_certificates: Dict[str, Optional[DeploymentConfigsProductionMTLSCertificates]]
    """mTLS bindings used for Pages Functions."""

    placement: DeploymentConfigsProductionPlacement
    """Placement setting used for Pages Functions."""

    queue_producers: Dict[str, Optional[DeploymentConfigsProductionQueueProducers]]
    """Queue Producer bindings used for Pages Functions."""

    r2_buckets: Dict[str, Optional[DeploymentConfigsProductionR2Buckets]]
    """R2 buckets used for Pages Functions."""

    services: Dict[str, Optional[DeploymentConfigsProductionServices]]
    """Services used for Pages Functions."""

    usage_model: Literal["standard", "bundled", "unbound"]
    """The usage model for Pages Functions."""

    vectorize_bindings: Dict[str, Optional[DeploymentConfigsProductionVectorizeBindings]]
    """Vectorize bindings used for Pages Functions."""

    wrangler_config_hash: str
    """Hash of the Wrangler configuration used for the deployment."""


class DeploymentConfigs(TypedDict, total=False):
    """Configs for deployments in a project."""

    preview: DeploymentConfigsPreview
    """Configs for preview deploys."""

    production: DeploymentConfigsProduction
    """Configs for production deploys."""


class SourceConfig(TypedDict, total=False):
    deployments_enabled: bool
    """
    Whether to enable automatic deployments when pushing to the source repository.
    When disabled, no deployments (production or preview) will be triggered
    automatically.
    """

    owner: str
    """The owner of the repository."""

    owner_id: str
    """The owner ID of the repository."""

    path_excludes: SequenceNotStr[str]
    """A list of paths that should be excluded from triggering a preview deployment.

    Wildcard syntax (`*`) is supported.
    """

    path_includes: SequenceNotStr[str]
    """A list of paths that should be watched to trigger a preview deployment.

    Wildcard syntax (`*`) is supported.
    """

    pr_comments_enabled: bool
    """Whether to enable PR comments."""

    preview_branch_excludes: SequenceNotStr[str]
    """A list of branches that should not trigger a preview deployment.

    Wildcard syntax (`*`) is supported. Must be used with
    `preview_deployment_setting` set to `custom`.
    """

    preview_branch_includes: SequenceNotStr[str]
    """A list of branches that should trigger a preview deployment.

    Wildcard syntax (`*`) is supported. Must be used with
    `preview_deployment_setting` set to `custom`.
    """

    preview_deployment_setting: Literal["all", "none", "custom"]
    """Controls whether commits to preview branches trigger a preview deployment."""

    production_branch: str
    """The production branch of the repository."""

    production_deployments_enabled: bool
    """Whether to trigger a production deployment on commits to the production branch."""

    repo_id: str
    """The ID of the repository."""

    repo_name: str
    """The name of the repository."""


class Source(TypedDict, total=False):
    """Configs for the project source control."""

    config: Required[SourceConfig]

    type: Required[Literal["github", "gitlab"]]
    """The source control management provider."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .details import (
    DetailsResource,
    AsyncDetailsResource,
    DetailsResourceWithRawResponse,
    AsyncDetailsResourceWithRawResponse,
    DetailsResourceWithStreamingResponse,
    AsyncDetailsResourceWithStreamingResponse,
)

__all__ = [
    "DetailsResource",
    "AsyncDetailsResource",
    "DetailsResourceWithRawResponse",
    "AsyncDetailsResourceWithRawResponse",
    "DetailsResourceWithStreamingResponse",
    "AsyncDetailsResourceWithStreamingResponse",
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CloudIntegrationEditParams"]


class CloudIntegrationEditParams(TypedDict, total=False):
    account_id: Required[str]

    aws_arn: str

    azure_subscription_id: str

    azure_tenant_id: str

    description: str

    friendly_name: str

    gcp_project_id: str

    gcp_service_account_email: str
